import hou
import XMLShapeFileReader
import copyPasteOps
import XMLShapeDump
import XMLShapeFileWriter
import houPrimGroups
import houMoveToBetterPosition
import houMoveToGoodPosition

class CopyPaste_XMLMannager:
	def __init__(self):
		self.fr = None

	def saveSelection(self, tree, gui, tagsInSourcePrim, SrcPrimSize, path, filename):
		guiparms = {}
		guiparms['guiparms'] = [['actualNode',gui.parm('actualNode').eval()],['primitiveNum',gui.parm('primitiveNum').eval()]]
		cpStatusparms = {}
		cpStatusparms['cpStatusparms'] = [['tagsInSourcePrim',tagsInSourcePrim],['SrcPrimSize',SrcPrimSize]]
		x = XMLShapeDump.DirectDumper(path,filename)
		x.visitAllCopiedNodes(tree, guiparms, cpStatusparms)
	
	def getCopiedTree(self, filepath, gui):
		self.fr = XMLShapeFileReader.buildXMLShapeFileReader(filepath)
		tree = [] #On es desara el resultat de la copia com si s'hagues fet directament amb Houdini
		#Creem una carpeta temporal dins de /obj anomenada XMLCLTemp on hi crearem tots els nodes
		#temporalment per a realitzar la copia en memoria.
		temp = hou.node('/obj').createNode('geo','XMLCLTemp')
		for n in temp.children():
			n.destroy()
		self.fr.initRules()
		while (not self.fr.finishedRules()):
			#Obtenim els filter
			pred = self.fr.getPredecessor()
			#Inicialitzem l'iterador per obtenir les accions
			self.fr.initActions()
			#Recuperem el tipus de node del qual es tracta
			type = self.fr.currentActionName().split(':')[1]
			#Creem el node i l'hi assignem el que sabem
			#print "TIPUS = "+type
			actual = temp.createNode(type)
			tree.append(actual)
			actual.parm('filter').set(pred)
			actual_name = str(self.fr.currentActionOldName())
			#print "OLD NAME = "+actual_name+" tipus = "+actual_name
			if actual.name() != actual_name:
				actual.setName(actual_name)
			actual = self.readCompulsoryParameters(actual)
			#Iterem sobre totes les accions que trobem
			while(not self.fr.finishedActions()):
				#S'obtenen tots els parametres del node d'aquesta accio
				p = self.fr.getParameters().items()
				for item in p:
					key = item[0]
					value = item[1]
					if(self.is_number(value)):
						value = float(value)
					#Si no, es un String
					actual.parm(key).set(value)
				prod = self.fr.getProducts()
				i = 0
				for item in prod:
					actual.parm('product'+str(i)).set(item)
				self.fr.selectNextAction()
			self.fr.selectNextRule()
		self.fr.initConnections()
		conit = self.fr.getConnections()
		for c in conit:
			src = hou.node('/obj/XMLCLTemp/'+c[1])
			dest = hou.node('/obj/XMLCLTemp/'+c[0])
			#print "Source = "+str(src)+" Dest = "+str(dest)+" len(c) ="+str(len(c))+" c ="+str(c)
			dest.setNextInput(src)
		houMoveToGoodPosition.move(hou.node('/obj/XMLCLTemp'))
		# Recuperem els parametres del GUI i del CoyPasteStatus
		self.fr.initGuiParms()
		guiparms = self.fr.getGuiParms()
		self.fr.initCPSParms()
		cpsparms = self.fr.getCPStatusParms()
		for i in cpsparms:
			if i[0]=='tagsInSourcePrim':
				copyPasteOps.CopyPasteStatus.tagsInSourcePrim = eval(i[1])
			if i[0]=='SrcPrimSize':
				copyPasteOps.CopyPasteStatus.SrcPrimSize = eval(i[1])
		for i in guiparms:
			if i[0]=='actualNode':
				gui.parm('actualNode').set(i[1])
			if i[0]=='primitiveNum':
				gui.parm('primitiveNum').set(i[1])
		return tree

	def readCompulsoryParameters(self, actual):
		type = actual.type().name()
		if type=='Subdiv':
			numDiv = int(self.fr.readCompulsoryParameters(['Divisions']).values()[0])
			actual.parm('Divisions').set(numDiv)
		if type=='Comp':
			numDiv = int(self.fr.readCompulsoryParameters(['components']).values()[0])
			actual.parm('components').set(numDiv)
		return actual

	def is_number(self, s):
		try:
			num = float(s)
			return True
		except ValueError:
			return False

	def getNeededAssets(self, tree):
		import bE_general
		pathList = {}
		for n in tree:
			if bE_general.is_Insert_node(n):
				pathList[n.name()] = n.parm('asset').eval()
		return pathList
		
	def copyAssetsToHIP(self, pathList):
		import os
		import shutil
		newpath = hou.expandString('$HIP')+"/CopiedAssets"
		if not os.path.exists(newpath):
			os.makedirs(newpath)
		for k, v in pathList.iteritems():
			shutil.copy(v,newpath)

	def changeAssetRoutes(self, tree):
		import os
		for n in tree:
			if n.type().name() == 'Insert':
				fileName = os.path.split(n.parm('asset').eval())[1]
				n.parm('asset').set(hou.expandString('$HIP')+'/CopiedAssets/'+fileName)