# =========================================================
# =========================================================
#    Graph rewriting operations
# =========================================================
# =========================================================
import bE_general

# =========================================================
#    Node reconnecting operations
# =========================================================
def reconnectInputs(oldNode, newNodes):
	print "   reconnecting inputs from: ", oldNode, " to: ", newNodes
	for connection in oldNode.inputConnections():
		node = connection.inputNode()
		print "      Changing connection from: " + node.name()
		for n in newNodes:
			n.setNextInput(node)    
		oldNode.setInput(connection.inputIndex(), None)

def reconnectOutputs(oldNode, newNode):
	print "   reconnecting outputs from: ", newNode, " to: ", oldNode
	for connection in oldNode.outputConnections():
		node = connection.outputNode()
		print "      Changing connection to: " + node.name(),
		node.setInput(connection.inputIndex(), newNode)

# =========================================================
#    Node grouping functions
# =========================================================

#Collects all children from a given node that are of a given type
def collect(container, nodeType):
	listOfNodes = []
	for n in container.children():
		if n.type().name() == nodeType:
			listOfNodes.append(n)
	return listOfNodes 

#Checks whether two set of filter lists are the same or not...
def equalFilters(candidateFilters, refFilters):
	candidateFilters.sort()
	refFilters.sort()
	return candidateFilters == refFilters

#Checks if two lists (or TUPLES) of nodes are equal to each other
#e.g. Node.inputs() is a tuple -> cannot use sort() to compare as in equalFilters...
def equalNodes(nodeList, refList):
	if len(nodeList) != len(refList):
		return False
	for i in nodeList:
		if i not in refList:
			return False
	return True

#Groups sets of nodes depending on whether they have the same ancestors and filters...
def groupNodes(container, nodeList):
	result = []
	for n in nodeList:
		# now we have to classify each node into a group
		candidateFilters = bE_general.effectiveFilters(container, n)
		classified = False
		for group in result:
			# Let's check whether n belongs to this group or not...
			if not classified: # to avoid waisted computations while using "for x in list" sructures...
				ref = group[0] # take any exemplar from the set. For instance, the first one...
				refFilters = bE_general.effectiveFilters(container, ref)
				if equalFilters(candidateFilters, refFilters) and equalNodes(n.inputs(), ref.inputs()):
					# if they match, group them togeter.
					classified = True
					group.append(n)
		if not classified:
			# n doesn't belong to any existing group: put it in a new one
			result.append([n])
	return result

#Given two sets of groups, returns a set formed by the groups in the
#first one without the groups in the second set.
def subtractGroups(allGroups, avoidGroups):
	result = []
	for gr in allGroups:
		found = False
		for avoid in avoidGroups:
			if equalNodes(gr, avoid):
				found = True
		if not found:
			result.append(gr)
	return result

