# -*- coding: utf-8 -*-
import nodeGetters

# ==================================================
# Conexions Selectors
# ==================================================
class connectionsSelector:
	pass
		
class ascendingConnexion(connectionsSelector):
	def connected(self, node):
		return node.inputs()
	
class descendingConnexion(connectionsSelector):
	def connected(self, node):
		return node.outputs()

# ==================================================
# nodeNavigator: base class for traversing nodes...
# ==================================================
class nodeIterator:
	def __init__(self, container):
		if container == None:
			self.parent = hou.selectedNodes()[0].parent()
		else:
			self.parent = container

class depthFirstSearchIterator(nodeIterator):
	def __init__(self, connectionsSelector, container, nodeList, preorder = True):
		nodeIterator.__init__(self, container)
		self.connectionsSelector = connectionsSelector
		self.startingNodes = nodeList
		self.preorder = preorder
	def navigate(self):
		self.visitedNodes = []
		for starting in self.startingNodes:
			for depth, n in self.visitNode(starting, 0):
				yield depth, n
	def visitNode(self, node, currDepth):
		#print "depthFirstSearchIterator::visitNode - ", node
		if node.path() not in self.visitedNodes and self.continuingCondition(node):
			self.visitedNodes.append(node.path())
			if self.preorder:
				yield currDepth, node
			for n in self.connectionsSelector.connected(node):
				if n: #in Houdini, there can be inputs pointing to "None"...
					#print "   connectionsSelector.connected:", n
					for newDepth, newNode in self.visitNode(n, currDepth+1):
						yield newDepth, newNode
			if not self.preorder:
				yield currDepth, node
	def continuingCondition(self, node):
		return True

def test():
	import hou
	container = hou.node('/obj/example')
	it = depthFirstSearchIterator(descendingConnexion(), container, nodeGetters.getRootNodes(container).get(), preorder = True)
	for depth, n in it.navigate():
		print('_'*depth + n.name())
