# -*- coding: utf-8 -*-

import hou

# ==================================================================
#  moveToBetterPosition: A function to accomodate all nodes in a
#     Houdini graph in a "nice" manner. We use pyDot 
#     (http://dkbza.org/pydot.html), which is a wrapper around
#     Graphviz (http://www.graphviz.org/). In particular, we use
#     *dot* a tool for``hierarchical'' or layered drawings of directed 
#     graphs. The layout algorithm aims edges in the same direction 
#     (top to bottom, or left to right) and then attempts to avoid 
#     edge crossings and reduce edge length.
# ==================================================================
import sys
sys.path.append('C:/Aplicaciones/pydot-1.0.3')
sys.path.append('C:/Program Files/Graphviz2.26.3')
sys.path.append('C:/Program Files/pyparsion-1.5.5')
import pydot

# ==================================================================
#  move: Parameters
#     container: the node containing the nodes to sort. e.g. a geo
#                or a subnet node
#     fileName: moveToBetterPosition uses a temporary file to
#               store a Graphviz intermediate file in 'plain' 
#               format. The provided name will be appended a 
#               ".plain" extension. The file will be created in
#               the current $HIP folder.
#               This file will NOT be deleted afterwards.
#     xFactor, yFactor: horizontal and vertical scaling factors.
#                       There is no predefined set of values that
#                       will satisfy all requirements.
#     createImage: Optionally, moveToBetterPosition will create
#                  an image file in .jpg format with the layout
#                  of the graph. The filename will be the one 
#                  provided with the '.jpg' extension.
# ==================================================================
def move(container, fileName = 'test_moveToBetterPosition', xFactor = 1, yFactor = 1, createImage = False):
	basePath = hou.expandString('$HIP')

	# let's first do a moveToGoodPosition pass, to guarantee
	# the right overall size for the network...
	# Not very nice, but useful to have it well scaled...
	for n in container.children():
		n.moveToGoodPosition()

	hminX = 1000
	hminY = 1000
	hmaxX = -1000
	hmaxY = -1000
	for n in container.children():
		if n.position()[0] < hminX:
			hminX = n.position()[0]
		if n.position()[0] > hmaxX:
			hmaxX = n.position()[0]
		if n.position()[1] < hminY:
			hminY = n.position()[1]
		if n.position()[1] > hmaxY:
			hmaxY = n.position()[1]

	graph = pydot.Dot(graph_type='digraph',fontname="Verdana")
	for aNode in container.children():
		graph.add_node(pydot.Node(aNode.name()))
		for node in aNode.outputs():
			graph.add_edge(pydot.Edge(aNode.name(), node.name()))
	plainFileName = basePath + "/"+ fileName + ".plain"
	graph.write_plain(plainFileName)
	if createImage:
		jpgFileName   = basePath + "/" + fileName + ".jpg"
		graph.write_jpeg(jpgFileName, prog='dot')
		print "Wrote JPG image to: ", jpgFileName

	file = open(plainFileName)
	nodePositions = {}
	for line in file:
		if line.split(" ")[0] == "node":
			nodeName = line.split(" ")[1].strip('"')
			nodePositions[nodeName] = ( eval(line.split(" ")[2]), eval(line.split(" ")[3]) )
	minX = 1000
	minY = 1000
	maxX = -1000
	maxY = -1000
	for n in nodePositions.keys():
		if nodePositions[n][0] < minX:
			minX = nodePositions[n][0]
		if nodePositions[n][0] > maxX:
			maxX = nodePositions[n][0]
		if nodePositions[n][1] < minY:
			minY = nodePositions[n][1]
		if nodePositions[n][1] > maxY:
			maxY = nodePositions[n][1]
	xRatio =  (hmaxX - hminX) / (maxX - minX) * xFactor
	yRatio =  (hmaxY - hminY) / (maxY - minY) * yFactor

	for n in container.children():
		coords = nodePositions[n.name()]
		n.setPosition(hou.Vector2((coords[0]-minX)*xRatio+hminX, (coords[1]-minY)*yRatio+hminY))

		