#loadOTL library
import hou

def loadedLibrary (librarypath) :
    for f in hou.hda.loadedFiles():
        if f.find ( librarypath ) > -1:
            return f
    return ''

def loadLibrary(library, sESystem='sEp'):
    sEp = __import__(sESystem)
    import hou
    print "loadLibrary", library, "from", sESystem
    if library in sEp.skylineEngineModuleOTLs:
        libraryPath = sEp.skylineEngineModuleOTLs[library]
    else:
        libraryPath = sEp.skylineEnginePath + 'otls/' + library + '.otl'
    if not loadedLibrary(libraryPath):
        print
        print    "libraryNode initialization:\n", \
            "  Loading Definitions in " + libraryPath + ":\n", \
            "  ", hou.hda.definitionsInFile(libraryPath)
        hou.hda.installFile(libraryPath)
    #else:
    #    print " The library is already loaded"
        
def loadLibraries(libraries):
    for l in libraries:
        loadLibrary(l)
        
#=========================================================
def isHDA(node):
    if (node.type().definition() != None):
        return True
    else:
        return False

def belongsToOTL(node, library, sESystem='sEp'):
    sEp = __import__(sESystem)
    if library in sEp.skylineEngineModuleOTLs:
        libraryPath = sEp.skylineEngineModuleOTLs[library]
    else:
        libraryPath = sEp.skylineEnginePath + 'otls/' + library + '.otl'
    #print "testing: ", libraryPath
    defs = hou.hda.definitionsInFile(libraryPath)
    return node.type().name() in map(lambda x: x.nodeTypeName(), defs)

def unlockHDA(node):
    if isHDA(node):
        node.allowEditingOfContents()

def recursiveUnlockHDA(node, validOTLPaths):
	for v in validOTLPaths:
		if belongsToOTL(node, v):
			unlockHDA(node)
	for n in node.children():
		recursiveUnlockHDA(n, validOTLPaths)

#==========================================================
# Some convenience functions
#==========================================================
def belongsToAnyLibrary(node, libraries):
	for library in libraries:
		if belongsToOTL(node, library):
			return True
	return False

#---eof
