import houPrimGroups
import productTree
import hou
import controller


def locateComp(geo):
	import houTopologicalSort
	L = houTopologicalSort.topologicalConnectionSort(geo)
	for node in L: 
		if node.type().name() == 'Comp':
			comp = node
			break
	return comp

def printState(GUI):
	print "Main: ", GUI.parm('mainComp').eval()
	print "actual:", GUI.parm('actualNode').eval(), "->", GUI.parm('primitiveNum').eval()


def initPrimitiveSelection(geometry, GUI):

	obj = hou.node("/obj")
	obj.createNode("geo","vizualizer")
	visual = hou.node("/obj/vizualizer")
	visual.children()[0].destroy()
   
	visual.createNode("merge","merger")
	merger = hou.node("/obj/vizualizer/merger")

	#Second geo crated
	childrens = geometry.children()
	for child in childrens:
		addNode(visual,child,merger)
	color = visual.createNode("color","colorizer")
	color.setFirstInput(merger)
	color.parm('colorr').setExpression('ch("'+GUI.parm('SelectionColorr').path()+'")')
	color.parm('colorg').setExpression('ch("'+GUI.parm('SelectionColorg').path()+'")')
	color.parm('colorb').setExpression('ch("'+GUI.parm('SelectionColorb').path()+'")')
	offset = visual.createNode("primitive", "offset")
	offset.parm('doxform').set(True)
	offset.parm('tx').setExpression('ch("'+GUI.parm('SelectionOffset').path()+'")*$NX')
	offset.parm('ty').setExpression('ch("'+GUI.parm('SelectionOffset').path()+'")*$NY')
	offset.parm('tz').setExpression('ch("'+GUI.parm('SelectionOffset').path()+'")*$NZ')
	offset.setFirstInput(color)
	offset.setDisplayFlag(1)
	offset.setRenderFlag(1)
			
def addNode(vizualizer,child,merger):
	vizualizer.createNode("object_merge","Object_merge_"+child.name())
	vizualizer.createNode("delete","Delete_"+child.name())
	delete = hou.node(vizualizer.path()+"/Delete_"+child.name())
	objectMerger = hou.node(vizualizer.path()+'/Object_merge_'+child.name())
	objectMerger.parm("objpath1").set(child.path())
	merger.insertInput(0,delete)
	delete.insertInput(0,objectMerger)
	delete.parm("negate").set(1)
		
def addPrim(p):
	vizualizer = hou.node("/obj/vizualizer")
	n = productTree.parentNode(p)
	delete = hou.node(vizualizer.path()+"/Delete_"+n.name())
	delete.parm("pattern").set(delete.parm("pattern").eval()+" "+str(p.number()))
		
def removePrim(p):
	vizualizer = hou.node("/obj/vizualizer")
	n = productTree.parentNode(p)
	delete = hou.node(vizualizer.path()+"/Delete_"+n.name())
	string = delete.parm("pattern").eval()
	string = string.replace(str(p.number()),"")
	string = string.replace("  "," ")
	delete.parm("pattern").set(string)

def init_Visualizer(GUI):
	geoPath = GUI.parm('buildingGeo').eval()
	if geoPath == '':
		hou.ui.displayMessage("Error no geometry selected",buttons=("OK",))
	else:
		geo = hou.node(geoPath)
		initPrimitiveSelection(geo, GUI)
		comp = locateComp(geo)
		primitives = productTree.startingPrims(comp)
		GUI.parm('actualNode').set(comp.path())
		GUI.parm('mainComp').set(comp.path())
		GUI.parm('primitiveNum').set(0)
		addPrim(primitives[0])
		
def next_Prim(GUI):
	actual = GUI.parm('primitiveNum').eval()
	node = hou.node(GUI.parm('actualNode').eval())
	primitives = productTree.startingPrims(node)
	#print "startingPrims(", node, ") =", primitives
	#========  Now, get the next sibling  ========
	nextPrim = productTree.nextSibling(primitives[actual])
	#print "nextSibling:",nextPrim, "(actual:",primitives[actual],")"
	if nextPrim == None:
		nextPrim = productTree.firstChild(primitives[actual])
	#========  ok, now let's visualize it  ========
	removePrim(primitives[actual])
	addPrim(nextPrim)
	GUI.parm('primitiveNum').set(nextPrim.number())
	#printState(GUI)
	
def prev_Prim(GUI):
	actual = GUI.parm('primitiveNum').eval()
	node = hou.node(GUI.parm('actualNode').eval())
	primitives = productTree.startingPrims(node)
	#print "siblings:", primitives
	#========  Now, get the previous sibling  ========
	prevPrim = productTree.previousSibling(primitives[actual])
	if prevPrim == None:
		prevPrim = productTree.lastChild(primitives[actual])
	#========  ok, now let's visualize it  ========
	removePrim(primitives[actual])
	addPrim(prevPrim)
	GUI.parm('primitiveNum').set(prevPrim.number())
	#printState(GUI)
	
def up_Prim(GUI):
	actual = GUI.parm('primitiveNum').eval()
	node = hou.node(GUI.parm('actualNode').eval())
	if node.path() != GUI.parm('mainComp').evalAsString():
		primitives = productTree.startingPrims(node)
		prim = productTree.parentPrim(primitives[actual])
		if prim != None:
			removePrim(primitives[actual])
			addPrim(prim)
			GUI.parm('actualNode').set(productTree.parentNode(prim).path())
			GUI.parm('primitiveNum').set(prim.number())
		else:
			hou.ui.displayMessage("No parent primitive for the selected one",buttons=("OK",))
	else:
		hou.ui.displayMessage("You can't go up anymore, because you are in the main Comp node",buttons=("OK",))
	#printState(GUI)	
	
def down_Prim(GUI):
	children = []
	actual = GUI.parm('primitiveNum').eval()
	node = hou.node(GUI.parm('actualNode').eval())
	primitives = productTree.startingPrims(node)
	children = productTree.childPrims(primitives[actual])
	
	#print(primitives)
	#print(children)
	if len(children) > 0:
		removePrim(primitives[actual])
		addPrim(children[0])
		GUI.parm('actualNode').set(productTree.parentNode(children[0]).path())
		GUI.parm('primitiveNum').set(children[0].number())
	else:
		hou.ui.displayMessage("No children for this primitive",buttons=("OK",))
	#printState(GUI)

def fromPrimSelection(GUI):
	import selectPrim
	actual = GUI.parm('primitiveNum').eval()
	node = hou.node(GUI.parm('actualNode').eval())
	primitives = productTree.startingPrims(node)
	removePrim(primitives[actual])

	prim = selectPrim.getSelectedBuildingPrim()
	addPrim(prim)
	GUI.parm('actualNode').set(productTree.parentNode(prim).path())
	GUI.parm('primitiveNum').set(prim.number())
	