import hou
import houTopologicalSort
import sys
import XMLShapeFileWriter

import bE_general

# ======================================================================================
#  XMLShape saving visitor
# ======================================================================================
class XMLShapeDumper:
	def __init__(self, filePath=hou.expandString('$HIP'), fileName='test.xml', network=None):
		print "XMLShapeDumper -> writing XMLShape file"
		if not network:
			self.parent = hou.selectedNodes()[0].parent()
		else:
			self.parent = network
		self.fileName = fileName
		self.filePath = filePath
		self.w = XMLShapeFileWriter.XMLShapeFileWriter('XMLShape:Building')
		self.w.setHeader({"useFinalMerge":"bE_sistematicNodeVisitors"})
		self.dispatchedNodes = []
	def visitAllNodes(self):
		print "Parent: ", self.parent.path()
		sortedNodes = houTopologicalSort.topologicalConnectionSort(self.parent)
		print "Going to visit the nodes...", sortedNodes
		for n in sortedNodes:
			self.dispatch(hou.node(n))
		self.postprocess()
		self.w.createFile(self.filePath,self.fileName)
		print "dumped to:"+ self.filePath + self.fileName
	def dispatchAction(self, aNode):
		if aNode.path() not in self.dispatchedNodes:
			self.dispatchedNodes.append(aNode.path())
			actionName = 'bE:' + aNode.type().name()
			parms = {}
			prods = []
			types = {}
			for p in aNode.parms():
				parmName = p.name()
				if parmName != 'filter' and parmName.find('labelparm') < 0:
					if parmName.find('product') >= 0: 
						prods.append(p.evalAsString())
					else:
						parms[parmName] = str(p.eval())
						types[parmName] = type(p.eval()).__name__
			print parms
			self.w.addAction(actionName, aNode.name(), parms, types, prods)
		else:
			raise hou.OperationFailed("XMLShapeDumper: ERROR! Already processed node " + aNode.path())

# ======================================================================================
#  DirectDumper saving visitor:
#    With our graph-based paradigm, there is no need to use each label only once,
#    so we can save each node directly when processed. Much simpler, right?
# ======================================================================================
class DirectDumper(XMLShapeDumper):
	def __init__(self, filePath=hou.expandString('$HIP'), fileName='test.xml', network=None):
		XMLShapeDumper.__init__(self, filePath, fileName, network)
		print "DirectDumper! -> No CGAShape compatibility"
	def postprocess(self, guiparms, cpstatusparms):
		#Emplena els camps que son propis de la GUI i del copyPasteStatus i que calen per a fer el paste.
		self.w.addGuiParms(guiparms)
		self.w.addCPStatusParms(cpstatusparms)
		
	def dispatch(self, aNode):
		print "DirectDumper VISITING: " + aNode.name() + ": " + aNode.type().name()
		if not bE_general.is_bE_node(aNode):
			self.dispatchNon_bE(aNode)
		else:
			inputFilter = aNode.parm('filter').evalAsString()
			if inputFilter == '':
				inputFilter = "none"
			self.w.addRule(inputFilter)
			self.dispatchAction(aNode)
		return True

	def setUpConnections(self, nodelist):
		self.w.addConnection()
		linkList = {}
		for n in nodelist:
			if bE_general.is_bE_node(n):
				#print "setUpConnections VISITING: " + n.name() + ": " + n.type().name()
				outputs = n.outputs()
				if linkList.get(n)==None:
					linkList[n] = []
				for item in outputs:
					if item in nodelist and bE_general.is_bE_node(item):
						linkList[n].append(item) 
						#actualOList.append(item)
				inputs = n.inputs()
				for item in inputs:
					if item in nodelist and bE_general.is_bE_node(item):
						if linkList.get(item)==None:
							linkList[item] = []
						linkList[item].append(n)
						#self.w.addLink(item, [n])
		self.w.addLinks(linkList)
			
	def visitAllCopiedNodes(self, nodelist, guiparms, cpstatusparms):
		print "Processing copied nodes"
		for n in nodelist:
			self.dispatch(n)
		self.setUpConnections(nodelist)
		self.postprocess(guiparms, cpstatusparms)
		self.w.createFile(self.filePath,self.fileName)
		print "dumped to:"+ self.filePath + self.fileName
	def dispatchNon_bE(self,aNode):
		if aNode.path() not in self.dispatchedNodes:
			self.dispatchedNodes.append(aNode.path())
		print "   " + aNode.name() + " is not buildingEngine  --> (" + aNode.type().name() + ")"
		if aNode.type().name() == 'AsianRoof':
			actionName = 'Lib:AsianRoof:AsianRoof'
			inputFilter = aNode.inputs()[0].parm('product0').eval()
			print ">> selector:", inputFilter
			parms = {}
			types = {}
			prods = [] #this will be empty
			for p in aNode.parms():
				parmName = p.name()
				if parmName != 'filter' and parmName.find('labelparm') < 0:
					if parmName.find('product') >= 0: 
						prods.append(p.evalAsString())
					else: 
						parms[parmName] = p.evalAsString()
						types[parmName] = type(p.eval()).__name__
			self.w.addRule(inputFilter)
			self.w.addAction(actionName, aNode.name(), parms, types, prods)			