# =================================================================
# =====       XMLShape File Reader                            =====
# =================================================================
#
# This class was created to isolate all about the XMLShape file
# format, so different sintaxis could be used. For instance, the
# original CGAShape sintax can be read, if you provide the parser, 
# of course! :-)

print "  XMLShapeFileReader module imported"

from xml.dom import minidom, Node
from string import split


verboseLevel_XMLShapeFileReader = 0   #0 = no verbose, 1 = debug info, 2 = verbose

class XMLShapeFileReader:
	verboseLevel_XMLShapeFileReader = 0

	def __init__(self, fileToRead):
		openedFile = open(fileToRead)
		s = self.clean(openedFile)
		self.XMLShapeFile = minidom.parseString(s)
		self.allRules = []
		self.currentRule = ''
		self.currentPos = 0
		self.currentSuc = ''
		self.currentActionPos = 0
		self.currentAction = ''
		self.allCon = []
		self.allGuiParms = []
		self.allCPSParms = []

	#Esborra tots els separadors no desitjats per a poder-hi iterar correctament
	def clean(self, fileToClean):
		foo = fileToClean.read()
		without_n = foo.replace('\n', '')
		without_t = without_n.replace('\t', '')
  
		return without_t

	# *****************************************
	#           Header Information
	# *****************************************
	def getHeaderType(self):
		node = self.XMLShapeFile.getElementsByTagName('header')[0]
		type = (node.attributes.getNamedItem('type').nodeValue).encode('latin')
		if verboseLevel_XMLShapeFileReader > 0:
			print 'XMLShapeFileReader.header: ', type
		return type
	def getHeaderConfigurations(self):
		headerNode = self.XMLShapeFile.getElementsByTagName('header')[0]
		confs = headerNode.getElementsByTagName('config')
		result = {}
		for c in confs:
			finalName = (c.attributes.getNamedItem('name').nodeValue).encode('latin')
			finalValue = (c.attributes.getNamedItem('value').nodeValue).encode('latin')
			result[finalName] = finalValue
		return result

	# *****************************************
	#           Defined Attributes
	# *****************************************
	def getAttr(self):
		attrDecl = self.XMLShapeFile.getElementsByTagName('attrDeclaration')
		if attrDecl:
			attrContainerNode = attrDecl[0]
			attrNodes = attrContainerNode.getElementsByTagName('attr')
			result = {}
			for attr in attrNodes:
				finalName = (attr.attributes.getNamedItem('name').nodeValue).encode('latin')
				finalValue = (attr.attributes.getNamedItem('value').nodeValue).encode('latin')
				if verboseLevel_XMLShapeFileReader > 0:
					print "FOUND " + finalName +\
						" = " + (attr.attributes.getNamedItem('value').nodeValue).encode('latin')
				result[finalName] = finalValue
			return result
		else:
			return None
			
	def getAttrList(self):
		attrDecl = self.XMLShapeFile.getElementsByTagName('attrDeclaration')
		if attrDecl:
			attrContainerNode = attrDecl[0]
			attrNodes = attrContainerNode.getElementsByTagName('attr')
			result = []
			for attr in attrNodes:
				finalName = (attr.attributes.getNamedItem('name').nodeValue).encode('latin')
				result.append(finalName)
			return result
		else:
			return None

	def __parserExtendedInformation (self, extinfo) :
		extended_info =  {}
		parameters = split(extinfo, ',')
		for parameter in parameters :
			p = split(parameter, '=')
			extended_info[p[0]] = p[1]
		return extended_info                 
	
	def getExtendedInformation(self, node):        
		return self.__parserExtendedInformation(node.attributes.getNamedItem('ext').nodeValue.encode('latin'))

	def getAttrExtendedInfo(self):
		attrContainerNode = self.XMLShapeFile.getElementsByTagName('attrDeclaration')[0]
		attrNodes = attrContainerNode.getElementsByTagName('attr')
		result = {}
		for attr in attrNodes:
			finalName = (attr.attributes.getNamedItem('name').nodeValue).encode('latin')
			if attr.attributes.getNamedItem('ext'):
				finalExtInfo = self.__parserExtendedInformation(attr.attributes.getNamedItem('ext').nodeValue.encode('latin'))
				if verboseLevel_XMLShapeFileReader > 0:
					print "FOUND " + finalName +\
						" = " + finalExtInfo
				result[finalName] = finalExtInfo
		return result

	# *****************************************
	#           Rules Iterator
	# *****************************************
	def getNumRules(self):
		count = 0;
		#for each <rule>
		self.allRules = self.XMLShapeFile.getElementsByTagName('rule')
		for rule in self.allRules:
			# Check for empty rules generated: tkngs patch!
			if rule.hasChildNodes():
				count += 1
		return count
	
	# Inicialitza l'iterador al principi de tot del fitxer
	def initRules(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.initRules()'
		self.currentPos = 0
		self.allRules = self.XMLShapeFile.getElementsByTagName('rule')
		self.currentRule = self.allRules[0]
		# Check for empty rules generated: tkngs patch!
		while (not self.currentRule.hasChildNodes()) and not self.finishedRules():
			self.currentPos += 1
			self.currentRule = self.allRules[currentPos]
			
	def initConnections(self):
		self.allCon = self.XMLShapeFile.getElementsByTagName('connections')
	
	def initGuiParms(self):
		self.allGuiParms = self.XMLShapeFile.getElementsByTagName('guiparms')

	def initCPSParms(self):
		self.allCPSParms = self.XMLShapeFile.getElementsByTagName('cpstatusparms')

	# Selecciona internament la seguent regla (no retorna res ni escriu res)
	def selectNextRule(self):
		self.currentPos += 1
		if not self.finishedRules():
			self.currentRule = self.allRules[self.currentPos]
		# Check for empty rules generated: tkngs patch!
		while (not self.currentRule.hasChildNodes()) and not self.finishedRules():
			self.currentPos += 1
			self.currentRule = self.allRules[self.currentPos]
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.selectNextRule() - new pos:', self.currentPos

	# Retorna True si no hi ha mes regles per iterar
	def finishedRules(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.finishedRules()'
		return self.currentPos >= len(self.allRules)

	def getPredecessor(self):
		return self.getName(self.currentRule.getElementsByTagName('predecessor')[0])

	def getName(self, node):
		return (node.attributes.getNamedItem('name').nodeValue).encode('latin')

	def getValue(self, node):
		return (node.attributes.getNamedItem('value').nodeValue).encode('latin')
		
	def getSource(self, node):
		return (node.attributes.getNamedItem('src').nodeValue).encode('latin')
	
	def getDestination(self, node):
		return (node.attributes.getNamedItem('dest').nodeValue).encode('latin')
		
	# *****************************************
	#           Action Iterator
	# *****************************************
	
	# Retorna el nombre d'accions de la regla actual escollida per l'iterador de rules.
	def getNumActions(self):
		count = 0;
		#for each <rule>
		self.currentSuc = self.currentRule.getElementsByTagName('successor')[0]
		for action in self.currentSuc.getElementsByTagName('action'):
			count += 1
		return count

	# Torna a reiniciar l'iterador per tal que apunti a la primera accio de la regla actual
	def initActions(self):
		self.currentSuc = self.currentRule.getElementsByTagName('successor')[0]
		self.currentActionPos = 0
		self.currentAction = self.currentSuc.getElementsByTagName('action')[0]

	# Retorna True si no hi ha mes accions per iterar, altrament retorna Fals.
	def finishedActions(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.finishedActions()'
		return self.currentActionPos >= len(self.currentSuc.getElementsByTagName('action'))

	# Itera a la seguent accio de la regla actual de manera interna (No retorna res ni escriu res)
	def selectNextAction(self):
		self.currentActionPos += 1
		if not self.finishedActions():
			self.currentAction = self.currentSuc.getElementsByTagName('action')[self.currentActionPos]
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.selectNextAction() - new pos:', self.currentActionPos

	# Retorna el nom de l'accio actual a la que apunta l'iterador.
	def currentActionName(self):
		return (self.currentAction.attributes.getNamedItem('name').nodeValue).encode('latin')

	def currentActionOldName(self):
		return (self.currentAction.attributes.getNamedItem('nodename').nodeValue).encode('latin')
	
	# *****************************************
	#           Parameters and Products
	# *****************************************
	
	# 
	def __getFinalName(self, result, finalName):
		if result.has_key(finalName):
			number = 1
			while result.has_key(finalName+str(number)):
				number += 1
			return finalName+str(number)
		else:
			return finalName

	# Retorna una llista de strings amb tots els noms dels parametres que conte la accio actual dins de la regla actual dels dos iteradors respectivament
	def getParameterNames(self):
		nodesList = self.currentAction.getElementsByTagName('param')
		result = []
		for node in nodesList:
			result.append(self.getName(node))
		return result

	# Retorna una llista de tuples de parelles de string nom:valor que conte l'accio actual dins de la regla actual dels dos iteradors respectivament
	def getParameters(self):
		nodesList = self.currentAction.getElementsByTagName('param')
		result = {}
		for node in nodesList:
			finalName = self.__getFinalName(result, self.getName(node))
			result[finalName] = self.getValue(node)
		return result

	# Retorna una llista de tuples de parelles de string desti:source que representa un enllas entre els dos nodes.
	def getConnections(self):
		nodesList = self.allCon[0].getElementsByTagName('param')
		result = []
		i = 0
		for node in nodesList:
			result.append([self.getDestination(node),self.getSource(node)])
			i+=1
			#finalName = self.getDestination(node)
			#result[finalName] = self.getSource(node)
		return result
		
	def getGuiParms(self):
		guiparmlist = self.allGuiParms[0].getElementsByTagName('param')
		guiparms = []
		for n in guiparmlist:
			name = (n.attributes.getNamedItem('name').nodeValue).encode('latin')
			value = (n.attributes.getNamedItem('value').nodeValue).encode('latin')
			guiparms.append([name,value])
		return guiparms

	def getCPStatusParms(self):
		cpsparmlist = self.allCPSParms[0].getElementsByTagName('param')
		cpsParms = []
		for n in cpsparmlist:
			name = (n.attributes.getNamedItem('name').nodeValue).encode('latin')
			value = (n.attributes.getNamedItem('value').nodeValue).encode('latin')
			cpsParms.append([name,value])
		return cpsParms

	# Retorna una llista de strings amb els noms dels productes que genera el node de la regla actual i l'accio actual dels dos iteradors respectivament.
	def getProducts(self):
		nodesList = self.currentAction.getElementsByTagName('product')
		result = []
		for node in nodesList:
			result.append( self.getName(node) )
		return result

	#******************************************************************************
	#                     hasParameterDefined
	#******************************************************************************
	def hasParameterDefined(self, pName):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'hasParameterDefined: Looking for parameter ', pName, ' in Node parameters'
		params = self.currentAction.getElementsByTagName('param')
		found = False
		for tryName in params:
			if self.getName(tryName) == pName:
				if verboseLevel_XMLShapeFileReader > 1:
					print 'Compulsory: Param found: ', pName
				found = True
		if verboseLevel_XMLShapeFileReader > 1:
			print 'hasParameterDefined: FOUND Values ->', found
		return found


	#******************************************************************************
	#                     readCompulsoryParameters
	#******************************************************************************
	def readCompulsoryParameters(self, parameterNames):
		params = self.currentAction.getElementsByTagName('param')
		results = {}
		for parName in parameterNames:
			if verboseLevel_XMLShapeFileReader > 1:
				print 'Compulsory: Looking for parameter ', parName, ' in Node parameters'
			found = False
			for tryName in params:
				nodeName =self.getName(tryName)
				if nodeName == parName:
					finalName = self.__getFinalName(results, nodeName)
					results[finalName] = self.getValue(tryName)
					if verboseLevel_XMLShapeFileReader > 1:
						print 'Compulsory: Param found: ', finalName, ' value: ', getValue(tryName)
					found = True
			if (not found):
				text = 'Compulsory: PARAMETER NOT FOUND: ' + parName + ' !!!'
				errorMsg(text)
		if verboseLevel_XMLShapeFileReader > 1:
			print 'Compulsory: FOUND Values ->', results
		return results

	#******************************************************************************
	#                     readOptionalParameters
	#******************************************************************************
	def readOptionalParameters(self, parameterList):
		params = self.currentAction.getElementsByTagName('param')
		results = {}
		for (pname, pvalue) in parameterList:
			if verboseLevel_XMLShapeFileReader > 1:
				print 'Optional: Looking for parameter ', pname, ' in Node parameters'
			found = False
			for tryName in params:
				nodeName =self.getName(tryName)
				if nodeName == pname:
					finalName = self.__getFinalName(results, nodeName)
					results[finalName] = self.getValue(tryName)
					if verboseLevel_XMLShapeFileReader > 1:
						print 'Optional: Param found: ', finalName, ' value: ', getValue(tryName)
					found = True
			if (not found):
				finalName = self.__getFinalName(results, pname)
				results[finalName] = pvalue
				if verboseLevel_XMLShapeFileReader > 1:
					print 'Optional: Param  not found: ', pname, ' value', pvalue
		if verboseLevel_XMLShapeFileReader > 1:
			print 'Optional: Final Values ->', results
		return results

def buildXMLShapeFileReader(fileToRead):
	print 'Creating buildXMLShapeFileReader'
	fileReader = XMLShapeFileReader(fileToRead)
	return fileReader


