# =================================================================
# =====       XMLShape File Writer                            =====
# =================================================================
print "  XMLShapeFileWriter module imported"

from xml.dom import minidom, Node
from string import split
import codecs

verboseLevel_XMLShapeFileWriter = 0

class XMLShapeFileWriter:
        def __init__(self, type):
			string="<?xml version=" + chr(34) + "1.0" + chr(34)+ " encoding="+chr(34) +"UTF-8"+chr(34)+"?>\n <CGAShape> \n </CGAShape>"
			self.XMLShapeFile = minidom.parseString(string)
			self.root = self.XMLShapeFile.childNodes[0]
			node = self.XMLShapeFile.createElement('header')
			node.setAttribute('type',type)
			if verboseLevel_XMLShapeFileWriter > 0:
					print 'XMLShapeFileWriter.header: ', type
			self.root.appendChild(node) 
			self.currentRule = None
			self.currentCon = None
			self.currentGuiParms = None

        def createFile(self,Path,name='newXMLShapeFile.xml'):
			"""
				It converts the internal structure into an XMLShape file and stores
				it in the Path and name given, if the file exists it will overwrite it.
			"""
			Path=Path+'/'+name
			f=codecs.open(Path,'w',encoding='utf-8')
			self.XMLShapeFile.writexml(f, indent="", addindent="   ", newl="\n")
			f.close()
        
        # *****************************************
        #           Defined Attributes
        # *****************************************
        def setHeader(self, configs):
			header = self.root.getElementsByTagName('header')[0]
			for c in configs.keys():
				v = configs[c]
				a = self.XMLShapeFile.createElement('config')
				a.setAttribute('name',c)
				a.setAttribute('value',v)
				header.appendChild(a)

        # *****************************************
        #           Defined Attributes
        # *****************************************
        def setAttr(self, attrList, extInfo):
			attrib = self.XMLShapeFile.createElement('attrDeclaration')
			for at in attrList.keys():
				a = self.XMLShapeFile.createElement('attr')
				a.setAttribute('name',at)
				a.setAttribute('type',type(attrList[at]).__name__)
				a.setAttribute('value',attrList[at])
				if at in extInfo:
					a.setAttribute('ext',extInfo[at])
				attrib.appendChild(a)
			self.root.appendChild(attrib)

	def addConnection(self):
			self.currentCon = self.XMLShapeFile.createElement('connections')
			self.root.appendChild(self.currentCon)

        # *****************************************
        #           Link Iterator
        # *****************************************
        def addLink(self, source, linkList):
			#add it to the current Connection at self.currentCon...
			for n in linkList:
				pa = self.XMLShapeFile.createElement('param')
				pa.setAttribute('dest',n.name())
				pa.setAttribute('src', source.name())
				self.currentCon.appendChild(pa)

	def addLinks(self, linkMap):
			#add it tot current Connection at self.currentCon...
			for src, dest in linkMap.iteritems():
				dest = list(set(dest))
				for node in dest:
					pa = self.XMLShapeFile.createElement('param')
					pa.setAttribute('dest',node.name())
					pa.setAttribute('src', src.name())
					self.currentCon.appendChild(pa)
				
        # *****************************************
        #           Rules Iterator
        # *****************************************
        def addRule(self, predecessor):
			#create a new rule and store a ref at self.currentRule...
			self.currentRule = self.XMLShapeFile.createElement('rule')
			pred = self.XMLShapeFile.createElement('predecessor')
			pred.setAttribute('name',predecessor)
			pred.appendChild(self.XMLShapeFile.createTextNode(''))
			succ = self.XMLShapeFile.createElement('successor')
			self.currentRule.appendChild(pred)
			self.currentRule.appendChild(succ)
			self.root.appendChild(self.currentRule)

        # *****************************************
        #           Action Iterator
        # *****************************************
        def addAction(self, actionName, nodeName, parms, types, products):
			#add it to the current rule at self.currentRule...
			action = self.XMLShapeFile.createElement('action')
			action.setAttribute('name',actionName)
			action.setAttribute('nodename',nodeName)
			succ = self.currentRule.getElementsByTagName('successor')[0]
			succ.appendChild(action)
			
			print "function addaction"
			print parms
			print "endfunction addaction"
			for parm in parms.keys():
				pa = self.XMLShapeFile.createElement('param')
				pa.setAttribute('name',parm)
				pa.setAttribute('type',types[parm])
				pa.setAttribute('value',parms[parm])
				action.appendChild(pa)
			for prod in products:
				pr = self.XMLShapeFile.createElement('product')
				pr.setAttribute('name',prod)
				action.appendChild(pr)

	def addGuiParms(self, guiparms):
		self.currentGuiParms = self.XMLShapeFile.createElement('guiparms')
		self.root.appendChild(self.currentGuiParms)
		for p in guiparms['guiparms']:
			pa = self.XMLShapeFile.createElement('param')
			pa.setAttribute('name',str(p[0]))
			pa.setAttribute('value',str(p[1]))
			self.currentGuiParms.appendChild(pa)

	def addCPStatusParms(self, cpStatusparms):
		self.currentCPStatusParms = self.XMLShapeFile.createElement('cpstatusparms')
		self.root.appendChild(self.currentCPStatusParms)
		for p in cpStatusparms['cpStatusparms']:
			pa = self.XMLShapeFile.createElement('param')
			pa.setAttribute('name',str(p[0]))
			pa.setAttribute('value',str(p[1]))
			self.currentCPStatusParms.appendChild(pa)
#=================================================================================
#  Some code to test everything is OK
#=================================================================================
if __name__ == "__main__":
	w = XMLShapeFileWriter('XMLShape:Building')
	# ==========================  Header !
	conf = {"useFinalMerge":"True", "libName":"testLib", "nodeName":"yetAnotherTestNode"}
	w.setHeader(conf)
	# ==========================  Atrributes !
	attr = {"boxHeight":"2", "testB":"False", "testExpr":"(attr:boxHeight: * 2) + 1", "testC":"4"}
	ext  = {"boxHeight":"folder:FolderA,type:float", "testB":"folder:FolderA,type:bool", "testExpr":"folder:FolderA,type:float"}
	w.setAttr(attr, ext)
	# ==========================  Now, all rules!
	w.addRule('none')
	# addAction(name, parms, prods) !!!
	w.addAction('testAction','NodeName1',{'sizex':'5','sizey':'3'},['testa','testb'])
	w.addAction('testAction2','NodeName2', {'parmA':'5','parmB':'3'},['test1','test2'])
	w.createFile('C:/Users/Xanatos/Desktop','test.xml')
