#===============================================================================	
#===============================================================================	
#  Copy&Paste operations...
#===============================================================================	
#===============================================================================	
import hou
import sEp
import productTree
import houPrimGroups
import houPrimOps
from collections import deque
	
def deselectAllNodes():
	selected = hou.selectedNodes()
	for n in selected:
		n.setSelected(False)

class CopyPasteStatus:
	tagsInSourcePrim = None
	tagsInTargetPrim = None
	SrcPrimSize = None
	TargetPrimSize = None
	quickSelection = True
	saveSelection = ''

def load_CopyFile(GUI):
	import os
	import CopyPaste_XMLMannager
	path = hou.parm('/obj/Interface/GUI/loadCopyFile').eval()
	if os.path.isfile(path):
		cp = CopyPaste_XMLMannager.CopyPaste_XMLMannager()
		tree = cp.getCopiedTree(path,GUI)
		cp.copyAssetsToHIP(cp.getNeededAssets(tree))	#Fa la copia de tots els nous assets en una carpeta $HIP/CopiedAssets/
		cp.changeAssetRoutes(tree)		#Canvia els paths dels Assets dels Inserts per a que els trobin quan es faci el paste
		hou.copyNodesToClipboard(tree)	#Faig el COPY
		hou.node('/obj/XMLCLTemp').destroy()
		hou.parm('/obj/Interface/GUI/paste').hide(False)
	else:
		hou.parm('/obj/Interface/GUI/loadCopyFile').set('')

def select_Geometry(GUI):
	import commonPrimAncestor
	print "="*50+"\n"+"="+" "*21+"SELECT"+" "*21+"=\n"+"="*50
	#First we visualize the building from the obj/geoNodeName/MergeNodeName that 
	#should contain the building so that the user can select the desired geometry.
	obj_childs = hou.node("/obj/").children()
	geo = None
	for item in obj_childs:
		if item.type().name()=="geo" and item.name()!="Interface" and item.isObjectDisplayed() and item.isSelectableInViewport():
			geo = item
			break
	#We assume that there is only one merge node to visualize the entire building into the geo node.
	merge = None
	if geo:
		for item in geo.children():
			if item.type().name()=="merge" or item.type().name()=="object_merge":
				merge = item
				break
	if merge:
		#Visualize it so that the user can select 
		merge.setCurrent(True,True)
		#Start the selection pipeline to get the common selected prim ancestor
		ca = commonPrimAncestor.startCommonSelection(CopyPasteStatus.quickSelection)
		print "CommonPrimAncestor FOUND: "+str(ca)+"\n"
		#It should be only one, next if is only precaution.. not necessary
		if len(ca)==1:
			GUI.parm('primitiveNum').set(ca[0].number())
			GUI.parm('actualNode').set(ca[0].geometry().sopNode().path())
			GUI.parm('buildingGeo').set(ca[0].geometry().sopNode().parent().path())
			hou.parm('/obj/Interface/GUI/copy').hide(False)
			print "Primitive selected: "+str(ca[0])+"\n"
		elif len(ca)>1:
			print "ERROR: Too many common ancestor in select_Geometry from copyPasteOps.py \n"
		else:
			print "ALERT: No common ancestor was found in select_Geometry from copyPasteOps.py \n"
		hou.node("/obj/Interface/GUI").setCurrent(True,False)
	else:
		print "ERROR: No merge object was found in select_Geometry from copyPasteOps.py \n"
#===============================================================================	
#  Copy operation...
#===============================================================================	
#Despres d'haver la sel.leccio d'origen copio tots aquells nodes que intervenen a la primitiva per la que treballem, creant aixi
#un subgraf que copia al "clipboard"
def copy_Selection(GUI):
	import os
	import CopyPaste_XMLMannager
	print "="*50+"\n"+"="+" "*22+"COPY"+" "*22+"=\n"+"="*50
	#visualizer = hou.node("/obj/vizualizer")
      
	print "actual:"+str(GUI.parm('actualNode').eval())+"->"+str(GUI.parm('primitiveNum').eval())+"\n"
	
	nodeY =  hou.node(GUI.parm('actualNode').eval())
	# We need to assign the entry node n_p a "unique" name so we can recover it
	# when pasting. The problem is that the original name might be already in use
	# in the target graph, so Houdini would rename n_p to the next free name+number
	# node name, making it very difficult to track...
	originalNodeYName = nodeY.name()
	
	primNum = GUI.parm('primitiveNum').eval()
	prim =  nodeY.geometry().prims()[primNum]
	CopyPasteStatus.tagsInSourcePrim = houPrimGroups.primGroups(prim)
	
	tree = []
	# Creates a sub-graph with all the nodes that deal with the primitives
	#we are interested in
	getSubTree(tree, prim)
	print "Seleted sub tree nodes:", tree
	for node in tree:
		node.setColor(hou.Color((0,1,0))) #Pinto de verd els que hem de copiar
		
	CopyPasteStatus.SrcPrimSize = computeSizes(prim)
	nodeY.setName('primitiveProducerNode-N_p')
	
	# Inici de la gravacio del sub-tree en format XML segons el path que l'usuari ha posat ##########################################
	filename = CopyPasteStatus.saveSelection.split('/')[-1]
	if filename:
		filepath = CopyPasteStatus.saveSelection.split(filename)[0]
	else:
		filepath = ''
	if os.path.isdir(filepath):
		cp = CopyPaste_XMLMannager.CopyPaste_XMLMannager()
		cp.saveSelection(tree,GUI,CopyPasteStatus.tagsInSourcePrim,CopyPasteStatus.SrcPrimSize,filepath,filename)
	#################################################################################################################################
	hou.copyNodesToClipboard(tree) #Faig el COPY
	
	#Let's undo our changes, just in case...
	#prop.destroy()
	nodeY.setName(originalNodeYName)
	print "originalNodeYName:", originalNodeYName
	printNeededAssets("Copy", tree)
	hou.parm('/obj/Interface/GUI/copy').hide(False)
	hou.parm('/obj/Interface/GUI/paste').hide(False)
	return tree
	
#===============================================================================	
#  Paste operation...
#===============================================================================	
#Un cop triat on volem afegir la geomtria copia ens disposem a fer el "paste" amb el pas intermig (el Glue)
def paste_Selection(GUI):
	print "="*50+"\n"+"="+" "*22+"PASTE"+" "*21+"=\n"+"="*50
	geoPath = GUI.parm('buildingGeo').eval()
	if geoPath == '':
		hou.ui.displayMessage("Error no geometry selected",buttons=("OK",))
	else:
		geo = hou.node(geoPath)
	
	print "Paste_Selection GEO = "+str(geo)
	
	nodeX = hou.node(GUI.parm('actualNode').eval())
	primID = GUI.parm('primitiveNum').eval()
	prim = nodeX.geometry().prims()[primID]	

	#Faig el GLUE creant els nodes necessaris (un exception i dos selectors)
	exceptionNode = geo.createNode("Exception")
	selectorA = geo.createNode("Selector", "SelectorTargetGraph")
	selectorB = geo.createNode("Selector", "SelectorSourceGraph")
	selectorA.parm("operation").set(0)
	selectorA.parm("filter").set("selected")
	selectorB.parm("operation").set(1)	
	selectorB.parm("filter").set("selected")
	connectNodes(exceptionNode,selectorA)
	connectNodes(exceptionNode,selectorB)

	CopyPasteStatus.tagsInTargetPrim = houPrimGroups.primGroups(prim)
	print "tagsInSourcePrim:", CopyPasteStatus.tagsInSourcePrim
	print "tagsInTargetPrim:", CopyPasteStatus.tagsInTargetPrim
	tags = ' '.join(CopyPasteStatus.tagsInTargetPrim)
	exceptionNode.parm("filter").set(tags)
	exceptionNode.parm("expression").set(str(primID))
	exceptionNode.parm("product0").set("selected")
	exceptionNode.parm("product1").set("nonSelected")
	
	reconnectOutputs(nodeX, selectorA)
	
	connectNodes(nodeX,exceptionNode)
	disconnectNode(geo, nodeX)
	connectNode(geo, selectorA)

	deselectAllNodes() #To be sure nothing else is selected before pasting...
	
	# ====== Let's PASTE the nodes!!! =====
	hou.pasteNodesFromClipboard(geo) 
	undoWrongConnections()	
	entryPasteNode = geo.glob('primitiveProducerNode-N_p*')[0] #Let's hope it is unique!!
	transferUsefulConnections(entryPasteNode, selectorB)
	entryPasteNode.destroy()
	fixProportions(prim)
	addRequiredSourceLabels(selectorB)	
	moveToGoodPositionNodes(geo)
	closeBuilding()
	if hasFinalBuildingMerge(geo):
		unableBadMergeEntries(geo)
	# Well... this is embarrassing... but some Houdini nodes (Subdiv, it seems) need
	# to be reminded to recompute... weird... We don't know why it happens, or better
	# said, why it doesn't happen! :-(
	forceUpdate()
	path = hou.parm('/obj/Interface/GUI/loadCopyFile').set('')
	hou.parm('/obj/Interface/GUI/paste').hide(True)
	hou.parm('/obj/Interface/GUI/copy').hide(True)

#===============================================================================	
#  Auxiliary functions
#===============================================================================	
def hasFinalBuildingMerge(geo):
	for n in geo.children():
		if n.type().name() == "object_merge" and n.name() == "finalBuilding":
			return True
	return False

def unableBadMergeEntries(geo):
	for n in geo.children():
		if n.type().name() == "object_merge":
			num = n.parm("numobj").eval()
			for pos in range(num):
				if n.parm("objpath"+str(pos+1)).eval().find('Exception')>-1 and n.parm("group"+str(pos+1)).eval().find('selected')>-1:
					n.parm("enable"+str(pos+1)).set(0)
				elif n.parm("objpath"+str(pos+1)).eval().find('Comp')>-1:
					n.parm("enable"+str(pos+1)).set(0)

def computeSizes(prim):
	bBox = houPrimOps.boundingBox(prim)
	
	xmin = bBox.minvec()[0]
	xmax = bBox.maxvec()[0]
	sizeX = abs(xmax - xmin)
	
	ymin = bBox.minvec()[1]
	ymax = bBox.maxvec()[1]
	sizeY = abs(ymax - ymin)
	
	zmin = bBox.minvec()[2]
	zmax = bBox.maxvec()[2]
	sizeZ = abs(zmax - zmin)
	
	return (sizeX, sizeY, sizeZ)

# getSubTree: Given a primitive (prim), gets all nodes that process any
# of its descendants. Stores the result in tree.
def getSubTree(tree, prim):
	# First, process the primitive's parent node
	if not productTree.parentNode(prim) in tree:
		tree.append(productTree.parentNode(prim))
	# Then any insert node from that prim
	childIns = productTree.childInserts(prim)
	for ins in childIns:
		if not ins in tree:
			tree.append(ins)
	# Now, recursively continue with the children...
	children = productTree.childPrims(prim)
	for p in children:
		getSubTree(tree, p)

def startTreeTest(tree):
	testTree(tree, 0)
		
def testTree(tree, i):
	import copy
	node = tree[0]
	i=i+1
	outs = node.outputs()
	next = copy.copy(tree)
	next.__delitem__(0)
	if len(outs)>0:
		print "--->"+' '*i+node.name()
		for item in outs:
			if item in next:
				print "outactual = "+str(item.name())
				testTree(next, i)
		
def undoWrongConnections():
	# unconnects all possible wrong connections that may result from a copy op:
	# if a node in the target graph has the same name as a node that conected with
	# the selected subgraph in the source graph, then Houdini creates a connection,
	# which obviously is wrong...
	for n in hou.selectedNodes():
		#isNodeY = False
		for connection in n.inputConnections():
			childNode = connection.inputNode()
			if not childNode in hou.selectedNodes():
				print "nodeX:", n.name()
				#print "Desconectar", childNode.name()
				n.setInput(connection.inputIndex(), None) #selectorB)
				#selectorB.parm("product0").set(n.parm("filter").eval())
				#isNodeY = True

def fixProportions(prim):
	propX = 0
	propY = 0
	propZ = 0

	CopyPasteStatus.TargetPrimSize = computeSizes(prim)
	print "Target 0: ", CopyPasteStatus.TargetPrimSize[0], "Src 0: ",CopyPasteStatus.SrcPrimSize[0]
	print "Target 1: ", CopyPasteStatus.TargetPrimSize[1], "Src 1: ",CopyPasteStatus.SrcPrimSize[1]
	print "Target 2: ", CopyPasteStatus.TargetPrimSize[2], "Src 2: ",CopyPasteStatus.SrcPrimSize[2]
#	props = (CopyPasteStatus.TargetPrimSize[0]/CopyPasteStatus.SrcPrimSize[0], \
#			CopyPasteStatus.TargetPrimSize[1]/CopyPasteStatus.SrcPrimSize[1], \
#			CopyPasteStatus.TargetPrimSize[2]/CopyPasteStatus.SrcPrimSize[2])
	props = (0, \
			0, \
			0)
	if CopyPasteStatus.SrcPrimSize[0] != 0:
		propX = (CopyPasteStatus.TargetPrimSize[0]/CopyPasteStatus.SrcPrimSize[0])
		
	if CopyPasteStatus.SrcPrimSize[1] != 0:
		propY = (CopyPasteStatus.TargetPrimSize[1]/CopyPasteStatus.SrcPrimSize[1])
		
	if CopyPasteStatus.SrcPrimSize[2] != 0:
		propZ = (CopyPasteStatus.TargetPrimSize[2]/CopyPasteStatus.SrcPrimSize[2])

	props = (propX, \
			propY, \
			propZ)

		
	print "props: (", props[0], ",", props[1], ",", props[2], ")"
	
	for n in hou.selectedNodes():
		if n.type().name() == "Repeat":
			print "Filter",n.parm("filter").eval()
			print n.name()
			adjustProportion(n, "value", props)
		elif n.type().name() == "Subdiv":
			items = n.parm("Divisions").eval()
			for i in range(items):
				adjustProportion(n, "value"+str(i), props)
		elif n.type().name() == "Insert":
			n.parm("tx").set(n.parm("tx").eval()*props[1])
			n.parm("ty").set(n.parm("ty").eval()*props[0])
			n.parm("tz").set(n.parm("tz").eval()*(props[0]+props[1])/2)
			if n.parm("relx").eval() == 0:
				n.parm("sx").set(n.parm("sx").eval()*props[1])
			if n.parm("rely").eval() == 0:
				n.parm("sy").set(n.parm("sy").eval()*props[0])
			if n.parm("relz").eval() == 0:
				n.parm("sz").set(n.parm("sz").eval()*(props[0]+props[1])/2)
 
def adjustProportion(n, parm, props):
	axis = n.parm("axis").eval()
	val = n.parm(parm).eval()
	if axis == 0:
		n.parm(parm).set(val*props[1])
	elif axis == 1:
		n.parm(parm).set(val*props[0])
	elif axis == 2:
		n.parm(parm).set(val*(props[0]+props[1])/2)
	
def transferUsefulConnections(entryNode, selectorB):
	for n in entryNode.outputs():
		nFilter = n.parm('filter').eval()
		if nFilter in CopyPasteStatus.tagsInSourcePrim:
			#this node takes a filter that is in the original prim 
			#tags, so it is a USEFUL connection...
			#let's keep it!
			n.setNextInput(selectorB)

def addRequiredSourceLabels(selectorB):
	import bE_general
	filterTags = set()	
	productTags = set()
	#for connection in selectorB.outputConnections():
	#	childNode = connection.outputNode()
	#	lostTags.append(childNode.parm("filter").eval())
	#	for prim in childNode.geometry().prims():
	#		lostTags = lostTags + houPrimGroups.primGroups(prim)
	for n in hou.selectedNodes():
		#if n.type().name() != "attribcast":
		filterTags.update(bE_general.filters(n))
		productTags.update(bE_general.products(n))
	requiredTags = filterTags - productTags		
		
	# Afegim els tags que existien al subgrup copiat pero no al graf de l'efici a modificar.
	#lostTags = list(set(lostTags))
	
	needAddTags = [item for item in requiredTags if not item in CopyPasteStatus.tagsInTargetPrim]
	
	geo = selectorB.parent()
	for tag in needAddTags:
		labeler = geo.createNode("Labeler")
		labeler.parm("filter").set("selected")
		labeler.parm("prefix").set(tag)
		labeler.parm("sufix").set(2)
		nodeAnt = selectorB.inputConnections()[0].inputNode()
		labeler.setInput(len(nodeAnt.outputConnections()), nodeAnt)
		selectorB.setInput(0, labeler)
		# labeler.setInput(len(exceptionNode.outputConnections()), selectorB)
		
	print "lostTags:", filterTags
	print "____________"
	print "CopyPasteStatus.tagsInTargetPrim", CopyPasteStatus.tagsInTargetPrim

def moveToGoodPositionNodes(geo):
	for n in hou.selectedNodes():
		n.moveToGoodPosition()

def closeBuilding():
	import bE_sistematicNodeVisitors
	c = bE_sistematicNodeVisitors.ClosingVisitor()
	c.visitAllNodes()

def forceUpdate():
	for n in hou.selectedNodes():
		if n.type().name() == 'Subdiv':
			n.parm('rel0').pressButton()

def printNeededAssets(text, tree):
	import bE_general
	#basePath = hou.expandString('$HIP')
	print "########################################"
	print "#  "+text+" - assets needed:"
	print "#"
	for n in tree:
		if bE_general.is_Insert_node(n):
			print "# ", n.parm('asset').eval()
	print "########################################"

def toggleQSelect(GUI):
	CopyPasteStatus.quickSelection = not CopyPasteStatus.quickSelection
	if(CopyPasteStatus.quickSelection): print "Quick Selection = On"
	else: print "Quick Selection = Off"

def setSavePath(GUI):
	CopyPasteStatus.saveSelection = hou.parm('/obj/Interface/GUI/saveCopyToFile').eval()

################ REWRITTING #################
#############################################

def connectNodes(nodeA, nodeB):
 	nodeB.setInput(0, nodeA)

def reconnectInputs(oldNode, newNode):  
    for connection in oldNode.inputConnections():
        node = connection.inputNode()
        newNode.setInput(connection.inputIndex(), node)

def reconnectOutputs(oldNode, newNode):
    for connection in oldNode.outputConnections():
        node = connection.outputNode()
        node.setInput(connection.inputIndex(), newNode)

def disconnectNode(container, node):
   for n in container.children():
        if n.type().name() == "object_merge":
            num = n.parm("numobj").eval()
            for pos in range(num):
                if n.parm("objpath"+str(pos+1)).eval() == node.path():
                    n.parm("enable"+str(pos+1)).set(0)

def connectNode(container, node):
   for n in container.children():
        if n.type().name() == "object_merge":
            num = n.parm("numobj").eval()
            n.parm("numobj").set(num+1)
            n.parm("enable"+str(num+1)).set(1)
            n.parm("objpath"+str(num+1)).set(node.path())