class PrimSelector:
	pass
	
#==========================================================
# BottomUpPrimSelector class...
# It tries to identify the asset primitive corresponding to
# the object_merge (or merge) node primitive selected by
# the user. This implies checking all primitives output by
# all terminal nodes (the ones in the object_merge)...
#==========================================================
class BottomUpPrimSelector(PrimSelector):
	def equalPrims(self, a, b):
		if len(a.vertices()) != len(b.vertices()):
			return False
		posb = map((lambda v: v.point().position()), b.vertices())
		posa = map((lambda v: v.point().position()), a.vertices())
		return posa == posb
		for v in a.vertices():
			pos = v.point().position()
			if pos not in posb:
				return False
		return True
	
	def checkPrim(self, p, node):
		for c in node.geometry().prims():
			if self.equalPrims(p,c):
				return c
		return None
	
	def identifyBuildingDetailPrim(self, p):
		import hou
		n = p.geometry().sopNode()
		if n.type().name() == 'object_merge':
			print "checking node:", n.name()
			for num in range(n.parm('numobj').eval()):
				entry = num + 1
				print "     against:", hou.node(n.parm('objpath'+str(entry)).eval()), "(", n.parm('objpath'+str(entry)).eval(), ")"
				candidate = self.checkPrim(p, hou.node( n.parm('objpath'+str(entry)).eval() ))
				if candidate:
					print "Found!!!"
					return candidate
			print "not found!!!"
		elif n.type.name() == 'merge':
			print "checking node:", n.name()
			for testNode in n.inputs():
				print "     against:", testNode.name()
				candidate = checkPrim(p, testNode)
				if candidate:
					print "Found!!!"
					return candidate
			print "not found!!!"
		else:
			return p

	def getBuildingPrim(self, detailPrim):
		import productTree
		buildingDetailPrim = self.identifyBuildingDetailPrim(detailPrim)
		return productTree.parentPrim(buildingDetailPrim)

#==========================================================
# TopDownPrimSelector class
# starts from the comp node and checks every primitive
# until it finds one that has no children and is parent 
# of the detailPrim...
#==========================================================
class TopDownPrimSelector(PrimSelector):
	def __init__(self, currentPrim):
		self.geo = self.__locateGeo(currentPrim)
		#Now the self.comp is ALWAYS a list that can contain one or more Comps.
		self.comp = self.__locateComp(self.geo)

	def __locateGeo(self, currentPrim):
		# It doesn't mutter if it is a Geo or a Subnetwork, the locateComp 
		# will look for Comp i every subnetwork child of the node returned by this function
		return currentPrim.geometry().sopNode().parent()
		
	def __locateComp(self, geo):
		import houTopologicalSort
		#First look for all subnetworks geo has.
		subnet = []
		L = houTopologicalSort.topologicalConnectionSort(geo)
		for node in L:
			if node.type().name()=="subnet":
				subnet.append(node)
		#Lets find out if there is a comp in the geo node before looking the subnets.
		comp = []
		print "__localeComp geo = "+str(geo)+"\n"
		for node in L: 
			if node.type().name() == 'Comp':
				comp = [node]
				break
		#If its not in geo, we must look at all possible subnetworks.
		if not comp:
			for item in subnet:
				comp += self.__locateComp(item)
		return comp
		
	def getBuildingPrim(self, detailPrim):
		#Returns a list with the node
		import productTree
		from collections import deque
		print "getBuildingPrim self.comp = "+str(self.comp)+"\n"
		for item in self.comp:
			remaining = deque(productTree.startingPrims(item))
			while remaining:
				elem = remaining.popleft()
				elemChildren = productTree.childPrims(elem)
				if not elemChildren and productTree.isParent(elem, detailPrim):
					return elem
				remaining.extend(elemChildren) #extends at right...
		return None

#==========================================================
#  Get the "detail" prim selected by the user. A "detail"
# prim is one on an asset (e.g. a triangle in a window), not
# a "real" bE prim...
#==========================================================
def getPointedPrim():
	import hou
	import toolutils
	viewer = toolutils.sceneViewer()
	geo = viewer.selectGeometry()
	g = hou.selectedNodes()[0].geometry()
	p = g.prims()[int(geo.mergedSelectionString())]
	return p

def getSelectedBuildingPrim():
	import sEp
	detailPrim = getPointedPrim()
	#selector = BottomUpPrimSelector()
	selector = TopDownPrimSelector(detailPrim)
	return selector.getBuildingPrim(detailPrim)
