import hou
import collections
# ===============================================
# ==      pre-processing stage  =================
# ===============================================
# It executes once for the scene to generate de svgs and pngs

def init():
#Create a temporal Geo with File node inside and return it. Also the temporal geo node created to deleted when finish.
	geo = hou.node('/obj/').createNode('geo')
	file = geo.children()[0] #we already have a file when we create a geo container...
	return [file,geo]

def deleteTemporalsGeos(tupla):
	#Elimina el que genera el metode init()
	tupla[1].destroy()
	
def getFileName(filepath):
#Takes the filepath, extract the file name and return it.
	name = filepath.split('/')[-1]
	return name.split('.')[0]

def generateSVG(n,targetNode, basePath = None):
#Takes every Insert node and process it
	import SVGFileWriter
	import os
	targetNode.parm('file').set(n.parm('asset').eval())
	SVGname = getFileName(targetNode.parm('file').eval()) + '.svg'
	if not basePath:
		basePath = hou.expandString('$HIP') + '/SVGs'
		if not os.path.exists(basePath):
			print "Creating new route defined by user!"
			os.makedirs(basePath)
	else:
		if not os.path.exists(basePath):
			print "Creating new route defined by user!"
			os.makedirs(basePath)
	SVGFileWriter.writeSVG(targetNode, SVGname, basePath)

def generateSVGs(container, basePath = None):
#Main method to execute
	import bE_general
	temptuple = init()
	targetNode = temptuple[0]
	tempGeo = temptuple[1]
	for n in container.children():
		if bE_general.is_Insert_node(n):	
			generateSVG(n,targetNode, basePath)
	deleteTemporalsGeos(temptuple)

def getImageSize(path):
	from xml.dom.minidom import parseString
	file = open(path,'r')
	data = file.read()
	file.close()
	dom = parseString(data)
	xmlTag = str(dom.getElementsByTagName('rect')[0].toxml())
	pairs = xmlTag.split(' ')
	h = 0
	w = 0
	for i in pairs:
		actual = i.split('=')
		if actual[0]=="height":
			h = float(str(actual[1]).strip('"'))
		elif actual[0]=="width":
			w = float(str(actual[1]).strip('"'))
	return [w,h]
	
def convert2PNGs(basePath = None):
	import sEp
	import os
	if not basePath:
		basePath = hou.expandString('$HIP')
	SVGSource = str(basePath) + '/SVGs'
	PNGDestination = str(basePath) + '/PNGs'
	fileListing = os.listdir(str(basePath)+'/SVGs')
	for file in fileListing:
		[width, height] = getImageSize(str(SVGSource)+'/'+str(file))
		#java -jar batik-rasterizer.jar -d -w <width> -h <height> <path>\PNGs  <path>\SVGs
		command = 'java -jar '+ str(sEp.batikModule) + '/batik-rasterizer.jar -w '+str(width)+' -h '+str(height)+' -d '+ PNGDestination +' '+ SVGSource+ '/' + str(file)
		os.system(command)

# ===============================================
# ==      processing stage         ==============
# ==    (A.K.A. rock'n roll stage) ==============
# ===============================================			
#Executed once to switch to high interactive mode

def getPNGPath(oldInsert):
	#Creates the path where the texture should be 
	return hou.expandString('$HIP')+ '/PNGs/' + getFileName(oldInsert.parm('asset').eval()) + '.png'
	
def fillParms(oldInsert, newInsertTex):
	#Copies the common parameters and change the route of the .bgeo on asset file with an image (png)
	newInsertTex.parm('filter').set(oldInsert.parm('filter').eval())
	newInsertTex.parm('product0').set(oldInsert.parm('product0').eval())
	newInsertTex.parm('uvquickshade1_texture_su').set(oldInsert.parm('sx').eval())
	newInsertTex.parm('uvquickshade1_texture_sv').set(oldInsert.parm('sy').eval())
	newInsertTex.parm('uvquickshade1_texture_sw').set(oldInsert.parm('sz').eval())
	newInsertTex.parm('uvquickshade1_texture').set(getPNGPath(oldInsert))

def replaceNodeByTexture(target,replacementTable):
	import graphRewriting
	ins = target.inputs()
	if len(ins)>0:
		#Create InsertTex node
		newInsert = hou.node(target.parent().path()).createNode('InsertTex')
		#All old insert inputs are reconnected to the InsertTex
		graphRewriting.reconnectInputs(target, [newInsert])
		#Register the change in a dictionary
		replacementTable[newInsert] = target
		#Filling the new inserTex node parameters with the old ones (changing geometry by texture)
		fillParms(target,newInsert)

def replaceGeometriesByTextures(container,replacementTable):
	import bE_general
	targetNode = init()
	for n in container.children():
		if bE_general.is_Insert_node(n):	
			replaceNodeByTexture(n,replacementTable)
	deleteTemporalsGeos(targetNode)

def interactiveON(container):
	#The format of replacement Table is Keys='newInsertTexs' and value='oldInserts'
	replacementTable = {}
	#container = hou.node('/obj/geo1');
	replaceGeometriesByTextures(container,replacementTable)
	for key in replacementTable.keys():
		print key, replacementTable[key]
	return replacementTable

# ===============================================
# ==      processing stage         ==============
# ==    (A.K.A. rock'n roll stage) ==============
# ===============================================
#Executed once to switch to low interactive mode, back to geometry

def is_InsertTex_node(node):
	if node:
		return (node.type().name() == 'InsertTex')
	else:
		return False

def replaceNodeByGeometry(target, replacementTable):
	import graphRewriting
	oldInsert = replacementTable[target]
	#All insertTex inputs are reconnected to the old Insert
	graphRewriting.reconnectInputs(target, [oldInsert])
	
def replaceTexturesByGeometry(container, replacementTable):
	targetNode = init()
	for n in container.children():
		if is_InsertTex_node(n):
			replaceNodeByGeometry(n,replacementTable)
	deleteTemporalsGeos(targetNode)

def interactiveOFF(container, replacementTable,newMerge=None):
	replaceTexturesByGeometry(container, replacementTable)
	newMerge.destroy()
	clearTextures(replacementTable)
	mergeNodes = listMergeNodes(container)
	for item in mergeNodes:
		item.setDisplayFlag(True)

# ===============================================
# Utility functions =============================
def listMergeNodes(container):
	mergeNodes = []
	for n in container.children():
		if n.type().name() == 'merge' or n.type().name() == 'object_merge':
			mergeNodes.append(n)
			#print n.name() + '\n'
	return mergeNodes

def clearTextures(replacementTable):
	#Destroy all InsertTex nodes created by the high interactive mode to clean the graph's generated dust
	for key in replacementTable.keys():
		key.destroy()

def findTheKey(replacementTable, name):
	for key, value in replacementTable.items():
		if value.name() == name:
			return key
	return None
			
def createNewTexturesMerge(container, replacementTable):
	m = listMergeNodes(container)
	#Creating new Object Merge
	newOM = container.createNode('object_merge')
	#Initialized transform: Into Specified Object
	newOM.parm('xformtype').set(2)
	#Active the view mode
	newOM.setDisplayFlag(True)
	#For all merge nodes founded, we read all the input nodes
	for n in m:
		parms = n.allParms()
		try:
			actual = parms.next()
			#All parms before number of objects are omited cause aren't interesting at all
			while actual.name()!='numobj':
				actual = parms.next()
			#Initializating merge inputs
			newOM.parm('numobj').set(actual.eval())
			pnum = 1
			#This compares the existings visible merged nodes with the insertTex created by us
			while True:
				actual = parms.next()
				#look if is enabled, if it's not, is not necessary to show it.
				if actual.eval()==1:
					newOM.parm('enable'+str(pnum)).set(actual.eval())
					#Getting the Object path, that must be switchet for our insertTex path
					actual = parms.next()
					#Take the name without the initial ../ that's commonly found in paths
					name = str(actual.eval())[3:]
					#Look for the insertTex that has been switched with founded node
					key = findTheKey(replacementTable, name)
					if not key:
						#If it hasn't been found, it will be added to visualize as in the original merge (Can be a Comp filter)
						newOM.parm('objpath'+str(pnum)).set(actual.eval())
					else:
						#Else it will be added switched by founded node
						newOM.parm('objpath'+str(pnum)).set(str('../')+key.name())
					#Next is the Group parm
					actual = parms.next()
					newOM.parm('group'+str(pnum)).set(actual.eval())
					#Next is Expand group
					actual = parms.next()
					newOM.parm('expand'+str(pnum)).set(actual.eval())
				else:
					try:
						while True:
							actual = parms.next()
							if actual.name().startswith('enable'):
								break
							else:
								pass
					except StopIteration:
						return newOM
				pnum+=1
		except StopIteration:
			return newOM

def findRootGeo():
	root = hou.node('/obj')
	for item in root.children():
		if item.type().name() == 'geo':
			return item

# Returns a list of subnets or geos thant must be processed separetly
def getFolderList(Node):
	result = []
	for item in Node.children():
		if item.type().name() == 'subnet':
			result.append(item)
	if result:
		return result
	else:
		return [Node]

# This is the generic method for preprocessing SVG2PNG textures
def preProcessBuilding():
	#Pre: The building has only ONE 'geo' node inside /obj/ folder. Inside this 'geo' there can be
	# many 'subnet' nodes for different sides of the building.
	containerList = getFolderList(findRootGeo())
	#Processing Insert Nodes to extract SVGs
	for item in containerList:
		generateSVGs(item)
	#Converting the SVGs to PNGs
	convert2PNGs()

#Generic turn On Interactive mode function
def turnOnInteractive():
	containerList = getFolderList(findRootGeo())
	switchInfo = []
	for item in containerList:
		replacementTable = interactiveON(item)
		newMerge = createNewTexturesMerge(item, replacementTable)
		switchInfo.append((item, replacementTable, newMerge))
	return switchInfo

#Generic turn Off Interactive mode function
def turnOffInteractive(erase):
	#erase parameter is the list returned by the turnOnRacInteractive Method.
	containerList = getFolderList(findRootGeo())
	for item in erase:
		interactiveOFF(item[0], item[1], item[2])
	
# ===============================================
# RACCOLETE MAIN TEST
def preProcessRac():
	#Specifying which folders we want to process
	containerR = hou.node('obj/Raccolet/rightWing')
	containerL = hou.node('obj/Raccolet/leftWing')
	#Specifying where we want to save the SVGs explicitely
	#SVG_basePathR = hou.expandString('$HIP')+'/SVGs/Right'
	#SVG_basePathL = hou.expandString('$HIP')+'/SVGs/Left'
	#Processing Insert Nodes to extract SVGs
	generateSVGs(containerR)
	generateSVGs(containerL)
	#Converting the SVGs to PNGs
	#PNG_basePathR = hou.expandString('$HIP')+'/PNGs/Right'
	#PNG_basePathL = hou.expandString('$HIP')+'/PNGs/Left'
	convert2PNGs()

def turnOnRacInteractive():
	containerR = hou.node('obj/Raccolet/rightWing')
	containerL = hou.node('obj/Raccolet/leftWing')
	replacementTableR = interactiveON(containerR)
	newMergeR = createNewTexturesMerge(containerR, replacementTableR)
	replacementTableL = interactiveON(containerL)
	newMergeL = createNewTexturesMerge(containerL, replacementTableL)	
	return [replacementTableR, newMergeR, replacementTableL, newMergeL]

def turnOffRacInteractive(erase):
#erase parameter is the list returned by the turnOnRacInteractive Method.
	containerR = hou.node('obj/Raccolet/rightWing')
	containerL = hou.node('obj/Raccolet/leftWing')
	interactiveOFF(containerR, erase[0],erase[1])
	interactiveOFF(containerL, erase[2],erase[3])
	